@@ -303,7 +303,6 @@ class NotebookClient(LoggingConfigurable):
     def reset_execution_trackers(self):
         """Resets any per-execution trackers.
         """
-        self.kc = None
         self.code_cells_executed = 0
         self._display_id_map = {}
         self.widget_state = {}
@@ -364,17 +363,33 @@ class NotebookClient(LoggingConfigurable):
         -------
         kc : KernelClient
             Kernel client as created by the kernel manager `km`.
+        kernel_id : string-ized version 4 uuid
+           The id of the started kernel.
         """
         resource_path = self.resources.get('metadata', {}).get('path') or None
         if resource_path and 'cwd' not in kwargs:
             kwargs["cwd"] = resource_path
 
-        if self.km.ipykernel and self.ipython_hist_file:
+        # if self.km is a MultiKernelManager, it doesn't have an ipykernel attribute
+        # so no extra_arguments can be passed
+        if hasattr(self.km, 'ipykernel') and self.km.ipykernel and self.ipython_hist_file:
             self.extra_arguments += ['--HistoryManager.hist_file={}'.format(self.ipython_hist_file)]
 
-        await ensure_async(self.km.start_kernel(extra_arguments=self.extra_arguments, **kwargs))
+        kernel_id = await ensure_async(self.km.start_kernel(extra_arguments=self.extra_arguments,
+                                       **kwargs))
 
-        self.kc = self.km.client()
+        # if self.km is not a KernelManager, it's probably a MultiKernelManager
+        try:
+            self.km.client
+            km = self.km
+        except AttributeError:
+            try:
+                km = self.km.get_kernel(kernel_id)
+            except AttributeError:
+                raise AttributeError('self.km={} has no client() or get_kernel() method, '
+                                     'what is this?'.format(self.km))
+
+        self.kc = km.client()
         await ensure_async(self.kc.start_channels())
         try:
             await ensure_async(self.kc.wait_for_ready(timeout=self.startup_timeout))
@@ -382,7 +397,7 @@ class NotebookClient(LoggingConfigurable):
             await self._async_cleanup_kernel()
             raise
         self.kc.allow_stdin = False
-        return self.kc
+        return self.kc, kernel_id
 
     start_new_kernel_client = run_sync(async_start_new_kernel_client)
 
@@ -412,11 +427,12 @@ class NotebookClient(LoggingConfigurable):
         """
         Context manager for setting up the kernel to execute a notebook.
 
-        The assigns the Kernel Manager (`self.km`) if missing and Kernel Client(`self.kc`).
+        This assigns the Kernel Manager (`self.km`) if missing and Kernel Client(`self.kc`).
 
         When control returns from the yield it stops the client's zmq channels, and shuts
         down the kernel.
         """
+        reset_kc = kwargs.pop('reset_kc', False)
         if self.km is None:
             self.start_kernel_manager()
 
@@ -425,17 +441,31 @@ class NotebookClient(LoggingConfigurable):
         try:
             yield
         finally:
-            await self._async_cleanup_kernel()
+            if reset_kc:
+                await self._async_cleanup_kernel()
 
     async def async_execute(self, **kwargs):
         """
         Executes each code cell.
 
+        Parameters
+        ----------
+        kwargs :
+            Any option for `self.kernel_manager_class.start_kernel()`. Because
+            that defaults to AsyncKernelManager, this will likely include options
+            accepted by `AsyncKernelManager.start_kernel()``, which includes `cwd`.
+            If present, `reset_kc` is passed to `self.async_setup_kernel`:
+            if True, the kernel client will be reset and a new one will be created
+            and cleaned up after execution (default: False).
+
         Returns
         -------
         nb : NotebookNode
             The executed notebook.
         """
+        reset_kc = kwargs.get('reset_kc', False)
+        if reset_kc:
+            await self._async_cleanup_kernel()
         self.reset_execution_trackers()
 
         async with self.async_setup_kernel(**kwargs):
