@@ -20,6 +20,7 @@ from ..exceptions import CellExecutionError
 import IPython
 from traitlets import TraitError
 from nbformat import NotebookNode
+from jupyter_client import KernelManager, MultiKernelManager
 from jupyter_client.kernelspec import KernelSpecManager
 from nbconvert.filters import strip_ansi
 from testpath import modified_env
@@ -400,6 +401,43 @@ def test_synchronous_setup_kernel():
     assert executor.kc is None
 
 
+def test_startnewkernel_with_kernelmanager():
+    nb = nbformat.v4.new_notebook()
+    km = KernelManager()
+    executor = NotebookClient(nb, km=km)
+    kc, kernel_id = executor.start_new_kernel_client()
+    # no kernel_id for a single kernel manager
+    assert kernel_id is None
+    # prove it initalized client
+    assert kc is not None
+    # since we are not using the setup_kernel context manager,
+    # cleanup has to be done manually
+    kc.shutdown()
+    km.cleanup()
+    kc.stop_channels()
+
+
+def test_startnewkernel_with_multikernelmanager():
+    nb = nbformat.v4.new_notebook()
+    km = MultiKernelManager()
+    executor = NotebookClient(nb, km=km)
+    kc, kernel_id = executor.start_new_kernel_client()
+    # a multi kernel manager always gives back an id to the started kernel
+    assert kernel_id is not None
+    # prove it initalized client
+    assert kc is not None
+    # since we are not using the setup_kernel context manager,
+    # cleanup has to be done manually
+    kc.shutdown()
+    km.cleanup(kernel_id)
+    km.remove_kernel(kernel_id)
+    # check that the kernel doesn't exist anymore
+    with pytest.raises(KeyError) as e_info:
+        km.get_kernel(kernel_id)
+    assert str(e_info.value) == "'Kernel with id not found: {}'".format(kernel_id)
+    kc.stop_channels()
+
+
 class TestExecute(NBClientTestsBase):
     """Contains test functions for execute.py"""
 
@@ -544,6 +582,29 @@ while True: continue
             else:
                 assert u"# üñîçø∂é".encode('utf8', 'replace') in str(exc.value)
 
+    def test_reset_kernel_client(self):
+        filename = os.path.join(current_dir, 'files', 'HelloWorld.ipynb')
+
+        with io.open(filename) as f:
+            input_nb = nbformat.read(f, 4)
+
+        executor = NotebookClient(
+            input_nb,
+            resources=self.build_resources(),
+        )
+
+        executor.execute()
+        # we didn't ask to reset the kernel client, a new one must have been created
+        kc = executor.kc
+        assert kc is not None
+        executor.execute()
+        # we didn't ask to reset the kernel client, the previously created one must have been reused
+        assert kc == executor.kc
+        executor.execute(reset_kc=True)
+        # we asked to reset the kernel client, the previous one must have been cleaned up,
+        # a new one must have been created and also cleaned up
+        assert executor.kc is None
+
     def test_custom_kernel_manager(self):
         from .fake_kernelmanager import FakeCustomKernelManager
 
